@@ -142,9 +142,14 @@ impl InstalledDist {
             // https://setuptools.pypa.io/en/latest/deprecated/python_eggs.html#egg-links
             // https://github.com/pypa/pip/blob/946f95d17431f645da8e2e0bf4054a72db5be766/src/pip/_internal/metadata/importlib/_envs.py#L86-L108
             let contents = fs::read_to_string(path)?;
-            let target = if let Some(line) = contents.lines().find(|line| !line.is_empty()) {
-                PathBuf::from(line.trim())
-            } else {
+            let Some(target) = contents.lines().find_map(|line| {
+                let line = line.trim();
+                if line.is_empty() {
+                    None
+                } else {
+                    Some(PathBuf::from(line))
+                }
+            }) else {
                 warn!("Invalid `.egg-link` file: {path:?}");
                 return Ok(None);
             };
@@ -168,7 +173,7 @@ impl InstalledDist {
                     return Ok(None);
                 }
             };
-            let metadata = match pypi_types::Metadata23::parse_pkg_info(&content) {
+            let metadata = match pypi_types::Metadata10::parse_pkg_info(&content) {
                 Ok(metadata) => metadata,
                 Err(err) => {
                     warn!("Failed to parse metadata for {path:?}: {err}");
@@ -178,7 +183,7 @@ impl InstalledDist {
 
             return Ok(Some(Self::LegacyEditable(InstalledLegacyEditable {
                 name: metadata.name,
-                version: metadata.version,
+                version: Version::from_str(&metadata.version)?,
                 egg_link: path.to_path_buf(),
                 target,
                 target_url: url,
@@ -279,6 +284,16 @@ impl InstalledDist {
             Self::LegacyEditable(dist) => Some(&dist.target_url),
         }
     }
+
+    /// Return true if the distribution refers to a local file or directory.
+    pub fn is_local(&self) -> bool {
+        match self {
+            Self::Registry(_) => false,
+            Self::Url(dist) => matches!(&*dist.direct_url, DirectUrl::LocalDirectory { .. }),
+            Self::EggInfo(_) => false,
+            Self::LegacyEditable(_) => true,
+        }
+    }
 }
 
 impl DistributionMetadata for InstalledDist {
