@@ -8,9 +8,11 @@ use rustc_hash::{FxHashMap, FxHashSet};
 use url::Url;
 
 use distribution_types::{
-    InstalledDist, Name, Requirement, UnresolvedRequirement, UnresolvedRequirementSpecification,
+    Diagnostic, InstalledDist, Name, Requirement, UnresolvedRequirement,
+    UnresolvedRequirementSpecification,
 };
 use pep440_rs::{Version, VersionSpecifiers};
+use pypi_types::VerbatimParsedUrl;
 use requirements_txt::EditableRequirement;
 use uv_cache::{ArchiveTarget, ArchiveTimestamp};
 use uv_interpreter::PythonEnvironment;
@@ -188,7 +190,7 @@ impl SitePackages {
     }
 
     /// Validate the installed packages in the virtual environment.
-    pub fn diagnostics(&self) -> Result<Vec<Diagnostic>> {
+    pub fn diagnostics(&self) -> Result<Vec<SitePackagesDiagnostic>> {
         let mut diagnostics = Vec::new();
 
         for (package, indexes) in &self.by_name {
@@ -201,7 +203,7 @@ impl SitePackages {
 
             if let Some(conflict) = distributions.next() {
                 // There are multiple installed distributions for the same package.
-                diagnostics.push(Diagnostic::DuplicatePackage {
+                diagnostics.push(SitePackagesDiagnostic::DuplicatePackage {
                     package: package.clone(),
                     paths: std::iter::once(distribution.path().to_owned())
                         .chain(std::iter::once(conflict.path().to_owned()))
@@ -218,7 +220,7 @@ impl SitePackages {
 
                 // Determine the dependencies for the given package.
                 let Ok(metadata) = distribution.metadata() else {
-                    diagnostics.push(Diagnostic::IncompletePackage {
+                    diagnostics.push(SitePackagesDiagnostic::IncompletePackage {
                         package: package.clone(),
                         path: distribution.path().to_owned(),
                     });
@@ -228,7 +230,7 @@ impl SitePackages {
                 // Verify that the package is compatible with the current Python version.
                 if let Some(requires_python) = metadata.requires_python.as_ref() {
                     if !requires_python.contains(self.venv.interpreter().python_version()) {
-                        diagnostics.push(Diagnostic::IncompatiblePythonVersion {
+                        diagnostics.push(SitePackagesDiagnostic::IncompatiblePythonVersion {
                             package: package.clone(),
                             version: self.venv.interpreter().python_version().clone(),
                             requires_python: requires_python.clone(),
@@ -246,7 +248,7 @@ impl SitePackages {
                     match installed.as_slice() {
                         [] => {
                             // No version installed.
-                            diagnostics.push(Diagnostic::MissingDependency {
+                            diagnostics.push(SitePackagesDiagnostic::MissingDependency {
                                 package: package.clone(),
                                 requirement: dependency.clone(),
                             });
@@ -261,11 +263,13 @@ impl SitePackages {
                                 )) => {
                                     // The installed version doesn't satisfy the requirement.
                                     if !version_specifier.contains(installed.version()) {
-                                        diagnostics.push(Diagnostic::IncompatibleDependency {
-                                            package: package.clone(),
-                                            version: installed.version().clone(),
-                                            requirement: dependency.clone(),
-                                        });
+                                        diagnostics.push(
+                                            SitePackagesDiagnostic::IncompatibleDependency {
+                                                package: package.clone(),
+                                                version: installed.version().clone(),
+                                                requirement: dependency.clone(),
+                                            },
+                                        );
                                     }
                                 }
                             }
@@ -338,9 +342,9 @@ impl SitePackages {
                             &requirement.extras,
                         ) {
                             let dependency = UnresolvedRequirementSpecification {
-                                requirement: UnresolvedRequirement::Named(
-                                    Requirement::from_pep508(dependency)?,
-                                ),
+                                requirement: UnresolvedRequirement::Named(Requirement::from(
+                                    dependency,
+                                )),
                                 hashes: vec![],
                             };
                             if seen.insert(dependency.clone()) {
@@ -360,7 +364,9 @@ impl SitePackages {
         while let Some(entry) = stack.pop() {
             let installed = match &entry.requirement {
                 UnresolvedRequirement::Named(requirement) => self.get_packages(&requirement.name),
-                UnresolvedRequirement::Unnamed(requirement) => self.get_urls(requirement.url.raw()),
+                UnresolvedRequirement::Unnamed(requirement) => {
+                    self.get_urls(requirement.url.verbatim.raw())
+                }
             };
             match installed.as_slice() {
                 [] => {
@@ -370,7 +376,7 @@ impl SitePackages {
                 [distribution] => {
                     match RequirementSatisfaction::check(
                         distribution,
-                        entry.requirement.source()?.as_ref(),
+                        entry.requirement.source().as_ref(),
                     )? {
                         RequirementSatisfaction::Mismatch | RequirementSatisfaction::OutOfDate => {
                             return Ok(SatisfiesResult::Unsatisfied(entry.requirement.to_string()))
@@ -402,9 +408,9 @@ impl SitePackages {
                             entry.requirement.extras(),
                         ) {
                             let dependency = UnresolvedRequirementSpecification {
-                                requirement: UnresolvedRequirement::Named(
-                                    Requirement::from_pep508(dependency)?,
-                                ),
+                                requirement: UnresolvedRequirement::Named(Requirement::from(
+                                    dependency,
+                                )),
                                 hashes: vec![],
                             };
                             if seen.insert(dependency.clone()) {
@@ -449,7 +455,7 @@ impl IntoIterator for SitePackages {
 }
 
 #[derive(Debug)]
-pub enum Diagnostic {
+pub enum SitePackagesDiagnostic {
     IncompletePackage {
         /// The package that is missing metadata.
         package: PackageName,
@@ -468,7 +474,7 @@ pub enum Diagnostic {
         /// The package that is missing a dependency.
         package: PackageName,
         /// The dependency that is missing.
-        requirement: pep508_rs::Requirement,
+        requirement: pep508_rs::Requirement<VerbatimParsedUrl>,
     },
     IncompatibleDependency {
         /// The package that has an incompatible dependency.
@@ -476,7 +482,7 @@ pub enum Diagnostic {
         /// The version of the package that is installed.
         version: Version,
         /// The dependency that is incompatible.
-        requirement: pep508_rs::Requirement,
+        requirement: pep508_rs::Requirement<VerbatimParsedUrl>,
     },
     DuplicatePackage {
         /// The package that has multiple installed distributions.
@@ -486,9 +492,9 @@ pub enum Diagnostic {
     },
 }
 
-impl Diagnostic {
+impl Diagnostic for SitePackagesDiagnostic {
     /// Convert the diagnostic into a user-facing message.
-    pub fn message(&self) -> String {
+    fn message(&self) -> String {
         match self {
             Self::IncompletePackage { package, path } => format!(
                 "The package `{package}` is broken or incomplete (unable to read `METADATA`). Consider recreating the virtualenv, or removing the package directory at: {}.", path.display(),
@@ -525,7 +531,7 @@ impl Diagnostic {
     }
 
     /// Returns `true` if the [`PackageName`] is involved in this diagnostic.
-    pub fn includes(&self, name: &PackageName) -> bool {
+    fn includes(&self, name: &PackageName) -> bool {
         match self {
             Self::IncompletePackage { package, .. } => name == package,
             Self::IncompatiblePythonVersion { package, .. } => name == package,
