@@ -7,12 +7,13 @@ use pubgrub::type_aliases::SelectedDependencies;
 use rustc_hash::{FxHashMap, FxHashSet};
 
 use distribution_types::{
-    Dist, DistributionMetadata, Name, ParsedUrlError, Requirement, ResolvedDist, VersionId,
+    Dist, DistributionMetadata, Name, Requirement, ResolutionDiagnostic, ResolvedDist, VersionId,
     VersionOrUrlRef,
 };
 use pep440_rs::{Version, VersionSpecifier};
 use pep508_rs::MarkerEnvironment;
-use uv_normalize::{ExtraName, PackageName};
+use pypi_types::{ParsedUrlError, Yanked};
+use uv_normalize::PackageName;
 
 use crate::dependency_provider::UvDependencyProvider;
 use crate::editables::Editables;
@@ -36,7 +37,7 @@ pub struct ResolutionGraph {
     /// The set of editable requirements in this resolution.
     pub(crate) editables: Editables,
     /// Any diagnostics that were encountered while building the graph.
-    pub(crate) diagnostics: Vec<Diagnostic>,
+    pub(crate) diagnostics: Vec<ResolutionDiagnostic>,
 }
 
 impl ResolutionGraph {
@@ -89,7 +90,7 @@ impl ResolutionGraph {
                             .unwrap_or_else(|| panic!("Every package should be pinned: {name:?}"))
                             .clone();
 
-                        diagnostics.push(Diagnostic::MissingExtra {
+                        diagnostics.push(ResolutionDiagnostic::MissingExtra {
                             dist,
                             extra: extra.clone(),
                         });
@@ -110,7 +111,7 @@ impl ResolutionGraph {
                         } else {
                             let dist = Dist::from_editable(name.clone(), editable.built.clone())?;
 
-                            diagnostics.push(Diagnostic::MissingExtra {
+                            diagnostics.push(ResolutionDiagnostic::MissingExtra {
                                 dist: dist.into(),
                                 extra: extra.clone(),
                             });
@@ -140,7 +141,7 @@ impl ResolutionGraph {
                         } else {
                             let dist = Dist::from_url(name.clone(), url_to_precise(url.clone()))?;
 
-                            diagnostics.push(Diagnostic::MissingExtra {
+                            diagnostics.push(ResolutionDiagnostic::MissingExtra {
                                 dist: dist.into(),
                                 extra: extra.clone(),
                             });
@@ -172,6 +173,23 @@ impl ResolutionGraph {
                         .expect("Every package should be pinned")
                         .clone();
 
+                    // Track yanks for any registry distributions.
+                    match dist.yanked() {
+                        None | Some(Yanked::Bool(false)) => {}
+                        Some(Yanked::Bool(true)) => {
+                            diagnostics.push(ResolutionDiagnostic::YankedVersion {
+                                dist: dist.clone(),
+                                reason: None,
+                            });
+                        }
+                        Some(Yanked::Reason(reason)) => {
+                            diagnostics.push(ResolutionDiagnostic::YankedVersion {
+                                dist: dist.clone(),
+                                reason: Some(reason.clone()),
+                            });
+                        }
+                    }
+
                     // Extract the hashes, preserving those that were already present in the
                     // lockfile if necessary.
                     let hashes = if let Some(digests) = preferences
@@ -393,8 +411,8 @@ impl ResolutionGraph {
             .map(|node| node.weight.dist)
     }
 
-    /// Return the [`Diagnostic`]s that were encountered while building the graph.
-    pub fn diagnostics(&self) -> &[Diagnostic] {
+    /// Return the [`ResolutionDiagnostic`]s that were encountered while building the graph.
+    pub fn diagnostics(&self) -> &[ResolutionDiagnostic] {
         &self.diagnostics
     }
 
@@ -494,8 +512,8 @@ impl ResolutionGraph {
                 .requires_dist
                 .iter()
                 .cloned()
-                .map(Requirement::from_pep508)
-                .collect::<anyhow::Result<_, _>>()?;
+                .map(Requirement::from)
+                .collect();
             for req in manifest.apply(requirements.iter()) {
                 let Some(ref marker_tree) = req.marker else {
                     continue;
@@ -573,35 +591,7 @@ impl From<ResolutionGraph> for distribution_types::Resolution {
                     )
                 })
                 .collect(),
+            graph.diagnostics,
         )
     }
 }
-
-#[derive(Debug)]
-pub enum Diagnostic {
-    MissingExtra {
-        /// The distribution that was requested with an non-existent extra. For example,
-        /// `black==23.10.0`.
-        dist: ResolvedDist,
-        /// The extra that was requested. For example, `colorama` in `black[colorama]`.
-        extra: ExtraName,
-    },
-}
-
-impl Diagnostic {
-    /// Convert the diagnostic into a user-facing message.
-    pub fn message(&self) -> String {
-        match self {
-            Self::MissingExtra { dist, extra } => {
-                format!("The package `{dist}` does not have an extra named `{extra}`.")
-            }
-        }
-    }
-
-    /// Returns `true` if the [`PackageName`] is involved in this diagnostic.
-    pub fn includes(&self, name: &PackageName) -> bool {
-        match self {
-            Self::MissingExtra { dist, .. } => name == dist.name(),
-        }
-    }
-}
