@@ -1,5 +1,6 @@
 //! Common operations shared across the `pip` API and subcommands.
 
+use pypi_types::{ParsedUrl, ParsedUrlError};
 use std::fmt::Write;
 use std::path::PathBuf;
 
@@ -9,17 +10,17 @@ use owo_colors::OwoColorize;
 use tracing::debug;
 
 use distribution_types::{
-    CachedDist, Dist, InstalledDist, Requirement, UnresolvedRequirementSpecification,
+    CachedDist, Diagnostic, InstalledDist, Requirement, ResolutionDiagnostic,
+    UnresolvedRequirementSpecification,
 };
 use distribution_types::{
     DistributionMetadata, IndexLocations, InstalledMetadata, InstalledVersion, LocalDist, Name,
-    ParsedUrl, RequirementSource, Resolution,
+    RequirementSource, Resolution,
 };
 use install_wheel_rs::linker::LinkMode;
 use pep440_rs::{VersionSpecifier, VersionSpecifiers};
 use pep508_rs::{MarkerEnvironment, VerbatimUrl};
 use platform_tags::Tags;
-use pypi_types::Yanked;
 use uv_cache::Cache;
 use uv_client::{BaseClientBuilder, RegistryClient};
 use uv_configuration::{
@@ -177,7 +178,7 @@ pub(crate) async fn resolve<InstalledPackages: InstalledPackagesProvider>(
     let python_requirement = PythonRequirement::from_marker_environment(interpreter, markers);
 
     // Map the editables to their metadata.
-    let editables = editables.as_metadata().map_err(Error::ParsedUrl)?;
+    let editables = editables.as_metadata();
 
     // Determine any lookahead requirements.
     let lookaheads = match options.dependency_mode {
@@ -286,17 +287,6 @@ pub(crate) async fn resolve<InstalledPackages: InstalledPackagesProvider>(
         .dimmed()
     )?;
 
-    // Notify the user of any diagnostics.
-    for diagnostic in resolution.diagnostics() {
-        writeln!(
-            printer.stderr(),
-            "{}{} {}",
-            "warning".yellow().bold(),
-            ":".bold(),
-            diagnostic.message().bold()
-        )?;
-    }
-
     Ok(resolution)
 }
 
@@ -521,10 +511,9 @@ pub(crate) async fn install(
         compile_bytecode(venv, cache, printer).await?;
     }
 
+    // Notify the user of any environment modifications.
     report_modifications(wheels, reinstalls, extraneous, printer)?;
 
-    report_yanks(&remote, printer)?;
-
     Ok(())
 }
 
@@ -666,8 +655,6 @@ fn report_dry_run(
         }
     }
 
-    report_yanks(&remote, printer)?;
-
     Ok(())
 }
 
@@ -721,39 +708,25 @@ pub(crate) fn report_modifications(
     Ok(())
 }
 
-/// Report on any yanked distributions in the resolution.
-pub(crate) fn report_yanks(remote: &[Dist], printer: Printer) -> Result<(), Error> {
-    // TODO(konstin): Also check the cache whether any cached or installed dist is already known to
-    // have been yanked, we currently don't show this message on the second run anymore
-    for dist in remote {
-        let Some(file) = dist.file() else {
-            continue;
-        };
-        match &file.yanked {
-            None | Some(Yanked::Bool(false)) => {}
-            Some(Yanked::Bool(true)) => {
-                writeln!(
-                    printer.stderr(),
-                    "{}{} {dist} is yanked.",
-                    "warning".yellow().bold(),
-                    ":".bold(),
-                )?;
-            }
-            Some(Yanked::Reason(reason)) => {
-                writeln!(
-                    printer.stderr(),
-                    "{}{} {dist} is yanked (reason: \"{reason}\").",
-                    "warning".yellow().bold(),
-                    ":".bold(),
-                )?;
-            }
-        }
+/// Report any diagnostics on resolved distributions.
+pub(crate) fn diagnose_resolution(
+    diagnostics: &[ResolutionDiagnostic],
+    printer: Printer,
+) -> Result<(), Error> {
+    for diagnostic in diagnostics {
+        writeln!(
+            printer.stderr(),
+            "{}{} {}",
+            "warning".yellow().bold(),
+            ":".bold(),
+            diagnostic.message().bold()
+        )?;
     }
     Ok(())
 }
 
 /// Report any diagnostics on installed distributions in the Python environment.
-pub(crate) fn report_diagnostics(
+pub(crate) fn diagnose_environment(
     resolution: &Resolution,
     venv: &PythonEnvironment,
     printer: Printer,
@@ -797,12 +770,9 @@ pub(crate) enum Error {
     #[error(transparent)]
     Lookahead(#[from] uv_requirements::LookaheadError),
 
-    #[error(transparent)]
-    ParsedUrl(Box<distribution_types::ParsedUrlError>),
-
     #[error(transparent)]
     Anyhow(#[from] anyhow::Error),
 
     #[error("Installed distribution has unsupported type")]
-    UnsupportedInstalledDist(#[source] Box<distribution_types::ParsedUrlError>),
+    UnsupportedInstalledDist(#[source] Box<ParsedUrlError>),
 }
